#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pigpiod_if.h>

#define TEMPSENSOR "/sys/devices/virtual/thermal/thermal_zone0/temp"

int temperature(void)
{
  static int rv = 0;
  FILE *fp;
  if ((fp = fopen(TEMPSENSOR, "r")))
  {
    char data[16];
    data[fread(data, 1, sizeof(data), fp)] = 0;
    fclose(fp);
    rv = atoi(data) / 1000;
  }
  return rv;
}

void usage(void)
{
  printf("Usage: pwnfan [options]\n");
  printf("\n");
  printf("  options: -c n ... cold temperature (*50 'C)\n");
  printf("           -h n ... hot  temperature (*60 'C)\n");
  printf("           -p n ... PWM output pin (12/*18 or 13/19)\n");
  printf("           -f n ... PWM Frequency (*10000 Hz)\n");
  printf("\n");
}

int main(int argc, char **argv)
{
  int pin   = 18;    // hardware PWM Pin
  int freq  = 10000; // hardware PWM frequency
  int cold  = 50;    // cold temperature
  int hot   = 60;    // hot temperature
  int status, temp, duty, c;

  printf("Cooling PWM Fan Controller Utility. Version 1.0\n");
  printf("Copyright (C) 2022 Sasapea's Lab. All right reserved.\n");
  printf("\n");

  while ((c = getopt(argc, argv, "f:c:h:p:")) != -1)
  {
    switch (c)
    {
      case 'f':
        freq = atoi(optarg);
        break;
      case 'c':
        cold = atoi(optarg);
        break;
      case 'h':
        hot = atoi(optarg);
        break;
      case 'p':
        switch((pin = atoi(optarg)))
        {
          case 12:
          case 13:
          case 18:
          case 19:
            break;
          default:
            usage();
            exit(1);
        }
        break;
      default:
        printf("\n");
        usage();
        exit(1);
    }
  }

  if (cold >= hot)
  {
    usage();
    exit(1);
  }

  printf("temperature range (%d - %d)\n", cold, hot);

  if ((status = pigpio_start(0, 0)))
    printf("Error: pigpio_start() = %d\n", status);
  else
  {
    while (1)
    {
      temp = temperature();
      if (temp <= cold)
        duty = 0;
      else if(temp >= hot)
        duty = 100;
      else
        duty = (temp - cold) * 100 / (hot - cold);
      hardware_PWM(pin, freq, duty * PI_HW_PWM_RANGE / 100);
      time_sleep(10);
    }
    pigpio_stop();
  }

  return 0;
}

